# Copyright 2014, 2019, 2020 Paul Hänsch
#
# This file is part of Confetti.
# 
# Confetti is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Confetti is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Confetti.  If not, see <http://www.gnu.org/licenses/>. 

check(){
  [ "$1" = "$2" ] && printf 'checked="checked"'
}

edit="$(GET e)"
order="$(GET o |grep -m1 -xE 'DOW|TOD')"

w_sort_courses(){
  cat <<-EOF
	[form .sort .search action="?" method="GET"
	  [span .label $(l10n sort_order):]
	  [radio "order" "DOW" $(check $order DOW) $(l10n order_DOW)]
	  [radio "order" "TOD" $(check $order TOD) $(l10n order_TOD)]
	  [submit "" "" $(l10n order_apply)]
	]
	EOF
}

cal_date(){
  { [ $# -eq 0 ] && cat || printf %s "$*"; } |sed -rnE '
    2q
    s/^([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2})([0-9]{2})([0-9]{2})Z$/\1-\2-\3 \4:\5:\6 UTC/p;t
    s/^TZID=(.+)\:([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2})([0-9]{2})([0-9]{2})$/TZ="\1" \2-\3-\4 \5:\6:\7/p;t
    s/^([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2})([0-9]{2})([0-9]{2})$/\1-\2-\3 \4:\5:\6/p;t
  '
}

cal_item(){
  local course="$1"
  local item cnt c
  shift 1

  for item in $@; do
    cnt="$(pdi_count "$course" "$item")"

    case $item in
      SUMMARY)
        printf '[h2 &shy%s]' "$(pdi_value "$course" SUMMARY)"
        ;;
      DTSTART)
        printf '[span .text .DTSTART %s %s ]' \
               "$(date -d "$(pdi_value "$course" DTSTART |cal_date)" '+%A %B %d, %Y - %H:%M')" \
               "$(l10n t_oclock)"
        ;;
      RRULE)
        dts_date="$(pdi_value "$course" DTSTART |cal_date)"
        rrule=" $(pdi_value "$course" RRULE)"
        rr_int="${rrule##*INTERVAL=}"; rr_int="${rr_int%%;*}"
        rr_count="${rrule##*COUNT=}"; rr_count="${rr_count%%;*}"
        rr_freq="${rrule##*FREQ=}"; rr_freq="${rr_freq%%;*}"
        rr_until="${rrule##*UNTIL=}"; rr_until="${rr_until%%;*}"
        rr_until="$(cal_date "${rr_until}")"

        [ "$rr_int" -eq 1 ] \
        && printf '[span .text .RRULE %s]' "$(l10n "s$rr_freq")" \
        || printf '[span .text .RRULE %s %s %s]' "$(l10n t_every)" "${rr_int}" "$(l10n $rr_freq)"
        case "$rrule $rr_freq" in
          *COUNT*DAILY*)
            printf '[span .text %s %s]' "$(l10n t_until)" "$(date -d "$dts_date + $((rr_int * rr_count)) day" "+%A %B %d, %Y - %H:%M")"
            ;;
          *COUNT*WEEKLY*)
            printf '[span .text %s %s]' "$(l10n t_until)" "$(date -d "$dts_date + $((rr_int * rr_count)) week" "+%A %B %d, %Y - %H:%M")"
            ;;
          *COUNT*MONTHLY*)
            printf '[span .text %s %s]' "$(l10n t_until)" "$(date -d "$dts_date + $((rr_int * rr_count)) month" "+%A %B %d, %Y - %H:%M")"
            ;;
          *COUNT*YEARLY*)
            printf '[span .text %s %s]' "$(l10n t_until)" "$(date -d "$dts_date + $((rr_int * rr_count)) year" "+%A %B %d, %Y - %H:%M")"
            ;;
          *UNTIL*)
            printf '[span .text %s %s]' "$(l10n t_until)" "$(date -d "$rr_until" "+%A %B %d, %Y - %H:%M")"
            ;;
        esac
        ;;
      attendance);;
      COMMENT)[ $cnt -gt 0 ] && printf '[h3 %s]' "$(l10n "$item")"
        seq 1 $cnt |while read c; do
          printf '[p .item .%s &shy;%s]' "$item" \
                 "$(pdi_value "$course" "$item" $c |unescape |HTML)"
        done
        ;;
      *)[ $cnt -gt 0 ] && printf '[h3 %s]' "$(l10n "$item")"
        seq 1 $cnt |while read c; do
          printf '[span .item .%s &shy;%s]' "$item" \
                 "$(pdi_value "$course" "$item" $c |unescape |HTML)"
        done
        ;;
    esac
  done
}

edit_item(){
  local course="$1"
  local item cnt c
  shift 1

  for item in $@; do
    cnt="$(pdi_count "$course" "$item")"
    [ "$cnt" -lt 1 ] && cnt=1

    case $item in
      DTSTART)
        local dtstart="$(pdi_value "$course" DTSTART |cal_date)"
        local ystart="${dtstart%%-*}"; ystart="${ystart##* }"
        local mstart="${dtstart#*-}"; mstart="${mstart%%-*}"
        local dstart="${dtstart##*-}"; dstart="${dstart%% *}"
        local hhstart="${dtstart##* }"; hhstart="${hhstart%%:*}"
        local mmstart="${dtstart##* }"; mmstart="${mmstart#*:}"; mmstart="${mmstart%:*}"
        local m mn cdow d

        cat <<-EOF
	[h3 . $(l10n DTSTART)]
	[input type="number" name="DTS_YEAR" value="${ystart}" placeholder="$(l10n YYYY)"]
	[select name="DTS_MONTH" onchange="this.form.submit();"
	$(m=1; for mn in $(l10n January February March April May June July August September October November December); do
	  printf '  [option value="%02i" %s . %s]\n' $m "$(selected $m $mstart)" "$mn"
	  m=$((m+1))
	done)
	][submit "DTS" "update" . $(l10n edit_dtscal)]
	[table .dtscalt
	  [tr $(printf '[th . %s]' $(l10n Mon Tue Wed Thu Fri Sat Sun))]
	  [tr $(
	local cdow d
	cdow="$(date -d ${ystart}-${mstart}-1 +%u)"
	seq 2 $cdow |xargs -n1 printf '[td .padding .%s]'
	d=1; while [ "$d" -lt 29 ] || [ "$(date -d ${ystart}-${mstart}-${d} +%m)" -eq "$mstart" ]; do
	  [ $cdow -eq 1 -a $d -ne 1 ] && printf ']\n  [tr '
	  printf '[td [input type="radio" name="DTS_DAY" #DTSCAL_%i value="%02i" %s][label for="DTSCAL_%i" %i]]' \
	         $d $d "$(checked $d $dstart)" $d $d
	  d=$((d + 1)); cdow=$(((cdow + 1) % 7))
	done 2>/dev/null
	)]
	]
	[span .DTSTIME $(l10n time):]
	[input type="number" name="DTS_HOUR" value="$hhstart" min="0" max="23"]:[input type="number" name="DTS_MINUTE" value="$mmstart" min="0" max="59"]
	EOF
        ;;
      RRULE)
        local dtstart="$(pdi_value "$course" DTSTART |cal_date)"
        local ystart="${dtstart%%-*}"; ystart="${ystart##* }"
        local mstart="${dtstart#*-}"; mstart="${mstart%%-*}"
        local dstart="${dtstart##*-}"; dstart="${dstart%% *}"

        local rrule="$(pdi_value "$course" RRULE)"
        local   rr_int="$(printf %s "$rrule" |sed -rn 's;^(.*\;[	 ]*)?INTERVAL=([0-9]+)(\;.*)?$;\2;p')"
        local rr_count="$(printf %s "$rrule" |sed -rn 's;^(.*\;[	 ]*)?COUNT=([0-9]+)(\;.*)?$;\2;p')"
        local  rr_freq="$(printf %s "$rrule" |sed -rn 's;^(.*\;[	 ]*)?FREQ=(DAILY|WEEKLY|MONTHLY|YEARLY)(\;.*)?$;\2;p')"
        local rr_until="$(printf %s "$rrule" |sed -rn 's;^(.*\;[	 ]*)?UNTIL=([0-9]{8}T[0-9]{6}Z)(\;.*)?$;\2;p')"
        local rr_uyear="${rr_until%????T??????Z}"
        local rr_umonth=${rr_until#????}; rr_umonth="${rr_umonth%??T??????Z}"
        local   rr_uday=${rr_until#??????}; rr_uday="${rr_uday%T??????Z}"
        local  rr_limit="ETERN"
        [ "$rr_count" ] && [ "$rr_count" -ge 0 ] && rr_limit="COUNT"
        [ "$rr_uyear" ] && [ "$rr_uyear" -ge 0 ] && rr_limit="UNTIL"

	cat <<-EOF
	[h3 . $(l10n "$item")]
	[span .item . $(l10n t_every)
	[input type="number" .RRULE .INTERVAL name="RRULE_INTERVAL" placeholder="#N" value="${rr_int:-1}" min="1"]
	[select .RRULE .FREQ name="RRULE_FREQ"
        $(for f in DAILY WEEKLY MONTHLY YEARLY; do
          printf '  [option value="%s" %s . %s]\n' "$f" "$(selected $f "$rr_freq")" "$(l10n $f)"
        done)
	]]
	[span .item [input type="radio" name="RRULE_LIMIT" value="ETERN" $(checked "$rr_limit" ETERN) . $(l10n t_eternal)][br]]
	[span .item
	[input type="radio" name="RRULE_LIMIT" value="COUNT" $(checked "$rr_limit" COUNT)]
	[input type="number" .RRULE .COUNT name="RRULE_COUNT" placeholder="#N" value="${rr_count:-1}" min="1"] $(l10n t_times)
	]
	[span .item
	[input type="radio" name="RRULE_LIMIT" value="UNTIL" $(checked "$rr_limit" UNTIL) . $(l10n t_until)]
	[input type="number" .RRULE .UYEAR name="RRULE_UYEAR" placeholder="$(l10n YYYY)" value="${rr_uyear:-$ystart}" min="$ystart"]
	[input type="number" .RRULE .UMONTH name="RRULE_UMONTH" placeholder="$(l10n MM)" value="${rr_umonth:-$mstart}" min="1" max="12"]
	[input type="number" .RRULE .UDAY name="RRULE_UDAY" placeholder="$(l10n DD)" value="${rr_uday:-$dstart}" min="1" max="31"]
	]
	EOF
        ;;
      COMMENT)
        printf '[h3 %s]' "$(l10n "$item")"
        seq 1 $cnt |while read c; do
          printf '[checkbox "%s_delete_%i" "true" .delete #%s_delete_%i][label for="%s_delete_%i" %s]' \
            "$item" $c "$item" $c "$item" $c "$(l10n delete)"
          printf '<textarea class="item %s" name="%s">%s</textarea>' \
            "$item" "$item" "$(pdi_value "$course" "$item" $c |unescape |HTML)"
        done
        printf '[button type="submit" name="action" value="addfield %s" %s ]' "$item" "$(l10n edit_addfield)"
        ;;
      attendance)
        printf '[h3 %s]' "$(l10n course_attendance)"
        for vcf in "$_DATA"/vcard/*.vcf; do
          fn="$(pdi_value "$(pdi_load "$vcf")" FN)"
          printf '%s/%s\n' "${vcf##*/}" "$fn"
        done \
        | sort -t/ -k2 \
        | while IFS=/ read -r vcf fn; do
          printf '[label [input type="checkbox" .item name="attendance" value="%s" %s] %s]' \
                 "$vcf" "$(grep -qxF "${coursefile##*/}	$vcf" "$_DATA/mappings/attendance" && printf 'checked="checked"')" "$fn"
        done
        ;;
      SUMMARY)
        printf '[h3 %s]' "$(l10n "$item")"
        printf '[input .item .%s name="%s" value="%s" placeholder="%s"]' \
               "$item" "$item" "$(pdi_value "$course" "$item" |unescape |HTML)" "$(l10n "$item")"
        ;;
      *)
        printf '[h3 %s]' "$(l10n "$item")"
        seq 1 $cnt |while read c; do
          printf '[checkbox "%s_delete_%i" "true" .delete #%s_delete_%i][label for="%s_delete_%i" %s]' \
            "$item" $c "$item" $c "$item" $c "$(l10n delete)"
          printf '[input .item .%s name="%s" value="%s" placeholder="%s"]' \
            "$item" "$item" "$(pdi_value "$course" "$item" $c |unescape |HTML)" "$(l10n "$item")"
        done
        printf '[button type="submit" name="action" value="addfield %s" %s ]' "$item" "$(l10n edit_addfield)"
        ;;
    esac
  done
}
